/** @file main.cpp contains the main program that a user would write.
 * see the documentation above "main"
 */
#include "mbed.h"               // ver 63
#include "SDFileSystem.h"       // ver 2, standard

// Modified standard components
#include "WiflyInterface.h"     // ver 4+, derived from version 4 with my mods

// My components
#include "SW_HTTPServer.h"      // ver 0, the initial release.
#include "Utility.h"

#include "DynamicPages.h"       // my dynamically generated pages

Serial pc(USBTX, USBRX);

LocalFileSystem local("local");         // some place to hold settings and maybe he static web pages
//SDFileSystem sd(p5, p6, p7, p8, "sd");  // for the static web pages


/// This section helps manage the credentials. I don't like the examples
/// that show clear text credentials, even if they are simply the 
/// access point ssid and passphrase. So, I created this to let me
/// edit a simple file on the file system and put them in there.
/// Marginally more secure, but at least not in the source code that
/// I might inadvertently share with others.
///
typedef struct
{
    char * ssid;
    char * pass;
} credentials;

#define CREDENTIAL_SIZE 50
bool ReadCredentials(char * file, credentials * cr)
{
    FILE *fp = fopen(file, "r");
    
    cr->ssid = (char *)malloc(CREDENTIAL_SIZE);
    cr->pass = (char *)malloc(CREDENTIAL_SIZE);
    if (fp) {
        fgets(cr->ssid, CREDENTIAL_SIZE, fp);
        fgets(cr->pass, CREDENTIAL_SIZE, fp);
        fclose(fp);
        RTrim(cr->ssid);
        RTrim(cr->pass);
        return true;
    } else {
        return false;
    }
}

void FreeCredentials(credentials * cr)
{
    // first secure them by wiping them out
    for (int i=0; i<CREDENTIAL_SIZE; i++) {
        cr->ssid[i] = cr->pass[i] = '*';
    }
    // then free the memory
    free(cr->ssid);
    free(cr->pass);
}


/// Smart-WiFly is a very primitive server using WiFly.
/// 
/// This is a working version, but it is not using the standardized wifly
/// library, which would not work for me... I had to make a number
/// of changes to get it to work well.
/// 
/// @caution If nothing visits its web page for a long time, it seems
/// to disable the interface. Perhaps a sleep timer I'm not spotting?
///
/// I created this because I couldn't find one that worked and wanted to
/// learn the WiFly module. There are a lot of possible improvements:
/// @li I think I'm not using the Socket interface as fully as I should.
/// @li I would like it to be non-blocking.
/// @li I would like it to be faster (the interface from mbed to wifly is 
///     limited to 230400 baud before it drops chars.
/// @li I would like to integrate this with the rtos.
/// @li If a page has multiple components (e.g. images), it appears 
///     unreliable. It doesn't see the request for the extra component.
///
/// history:
/// @li 20130602 added .txt to the supported types (e.g. robots.txt), so
///         revised the credentials to .crd, which is an unsupported type
///         so won't be delivered to the user.
///
/// @note Copyright &copy; 2013 by Smartware Computing, all rights reserved.
///     Individuals may use this application for evaluation or non-commercial
///     purposes. Within this restriction, changes may be made to this application
///     as long as this copyright notice is retained. The user shall make
///     clear that their work is a derived work, and not the original.
///     Users of this application and sources accept this application "as is" and
///     shall hold harmless Smartware Computing, for any undesired results while
///     using this application - whether real or imagined.
///
/// @author David Smart, Smartware Computing
///
int main() {
    credentials cr;     // handles the credentials
    
    pc.baud(460800);    // I like a snappy terminal, so crank it up!
    
    pc.printf("\r\nSmart WiFly 4 - build " __DATE__ " " __TIME__ "\r\n");

    if (!ReadCredentials("/local/user.crd", &cr)) {
        pc.printf("**** ERROR, no /local/user.crd file was found. ****\r\n");
        pc.printf("     awaiting watchdog (that means you push the button).\r\n");
        while(1);
    }
    
    // instantiate the WiflyInterface, then release the credentials
    WiflyInterface wifly(p28, p27, p23, p24, cr.ssid, cr.pass, WPA);
    FreeCredentials(&cr);

    // start it up as a client of my network    
    wifly.init(); // use DHCP
    
    wifly.baud(230400);     // I like this fast, but it drops chars at 460800
    
    while (!wifly.connect()) {
        printf(" ... failed to connect, retrying.\r\n");
        wifly.reset();
        wait(1.0);
    }
    printf("IP Address is %s\r\n", wifly.getIPAddress());

    // Now let's instantiate the web server - along with a few settings
    // among which is the file system path to the static pages.
    #define HTTP_SERVER_PORT 80
    HTTPServer svr(&wifly, HTTP_SERVER_PORT, "/local/", 30, 10, &pc);
    
    // But for even more fun, I'm registering a few dynamic pages
    // which you see the handlers for din DynamicPages.cpp. 
    // Here you can see the path to place on the URL.
    // ex. http://192.168.1.140/dyn
    svr.RegisterHandler("/dyn",  SuperSimpleDynamicPage);
    svr.RegisterHandler("/dyn1", SimpleDynamicPage);
    svr.RegisterHandler("/dyn2", SimpleDynamicForm);
    
    // Let the human know it is ready - if they are watching
    pc.printf("Waiting for a connection...\r\n");
    while (true)
        {
        svr.Poll();   // unfortunately, this is a blocking process
        //pc.printf("this %d, longest %d\r\n", thisRun, longestRun);
        }
}


